/*
 *  ======== SMPExample2.c ========
 *  This test creates 2 timers and maps their interrupt to Core0 and Core1 respectively.
 *  Once both timers expire, the application exits. The test also demonstrates Hwi logging.
 *
 */


/*
 *  ======== Header Files ========
 */
#include <stdlib.h>

#include <xdc/std.h>
#include <xdc/cfg/global.h>
#include <xdc/runtime/System.h>

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/hal/Core.h>
#include <ti/sysbios/hal/Timer.h>
#include <ti/sysbios/knl/Swi.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/smp/Load.h>
#include <ti/sysbios/knl/Semaphore.h>
#include <ti/sysbios/hal/Core.h>
#include <xdc/runtime/Timestamp.h>

/*
 *  ======== Global Variable Declaration ========
 */

Task_Handle tsk0;
Semaphore_Handle sem0;
Timer_Handle timer0, timer1;

/*
 *  ======== Macro definitions ========
 */

#define TIMER0_PERIOD	100000 // 1000
#define TIMER1_PERIOD	200000 // 1000

/*
 *  ======== Function Prototypes ========
 */

Void isr0Fxn(UArg arg);
Void myTask0(UArg arg0, UArg arg1);

/*
 * ======== main() begins here ========
 */
Int main(Int argc, char* argv[])
{
    Task_Params tskParams;
    Timer_Params timerParams;
    Semaphore_Params semParams;

    System_printf("Beginning test.\n");

    Task_Params_init(&tskParams);
    tskParams.priority = 2;
    tskParams.arg0 = 0;
    tskParams.affinity = 0;
    tsk0 = Task_create(myTask0, &tskParams, NULL);

    Timer_Params_init(&timerParams);
    timerParams.arg = 0;
    timerParams.periodType = Timer_PeriodType_COUNTS;
    timerParams.runMode = Timer_RunMode_ONESHOT;
    timerParams.startMode = Timer_StartMode_USER;
    timer0 = Timer_create(TIMER0, isr0Fxn, &timerParams, NULL);

    timerParams.arg = 1;
    timer1 = Timer_create(TIMER1, isr0Fxn, &timerParams, NULL);

    Semaphore_Params_init(&semParams);
    sem0 = Semaphore_create(0, &semParams, NULL);

    BIOS_start();
    return (0);
}

/*
 *  ======== isr0Fxn ========
 */
Void isr0Fxn(UArg arg)
{
	if (arg == 0) {
		System_printf("Core %d: Timer0 Expired!\n", Core_getId());
		Semaphore_post(sem0);
	}
	else if (arg == 1) {
		System_printf("Core %d: Timer1 Expired!\n", Core_getId());
		Semaphore_post(sem0);
	}
}

/*
 *  ======== myTask0 ========
 */
Void myTask0(UArg arg0, UArg arg1)
{
	/* Start Timer0 and Timer1 */
    Timer_trigger(timer0, TIMER0_PERIOD);
    Timer_trigger(timer1, TIMER1_PERIOD);

    while (Semaphore_getCount(sem0) != 2);

    BIOS_exit(0);
}
